using System;
using UnityEngine;
using UnityEngine.InputSystem;
using UnityEngine.Serialization;

public class BallBehavior : MonoBehaviour
{
    private Collider _sphereCollider;
    private Rigidbody _sphereRigidbody;
    [SerializeField] private float jumpHeight = 10f;
    private PlayerInput _playerInput;
    private InputAction _horizontalAction;
    private float _horizontalAxis;
    public int moveForwardSpeed = 3;
    public float keyboardMoveSpeed = 15f;
    private int _isKeyboard = 0;

    void Awake()
    {
        _playerInput = GetComponent<PlayerInput>();
        _horizontalAction = _playerInput.currentActionMap.FindAction("Horizontal");
        _horizontalAction.performed += moveActionPerformed;
        _horizontalAction.canceled += _ => _horizontalAxis = 0;
    }

    void Start()
    {
        _sphereCollider = GetComponent<Collider>();
        _sphereRigidbody = GetComponent<Rigidbody>();
        _sphereCollider.isTrigger = true;
        _sphereRigidbody.useGravity = true;

        var boxCollider = gameObject.AddComponent<SphereCollider>();
        boxCollider.isTrigger = false;
        boxCollider.radius = _sphereCollider.bounds.size.x / 2;
    }

    void Update()
    {
        Vector3 position = transform.position;
        float deltaTime = Time.deltaTime;
        position += Vector3.forward * (moveForwardSpeed * deltaTime);
        position += Vector3.right * deltaTime * _horizontalAxis;
        transform.position = position;
    }

    private void OnTriggerEnter(Collider other)
    {
        _sphereRigidbody.velocity = Vector3.up * jumpHeight;
        Debug.Log("Ball is now triggered");
    }

    private void moveActionPerformed(InputAction.CallbackContext context)
    {
        if (context.control.device.displayName.Contains("Keyboard"))
        {
            _isKeyboard = 1;
        }
        else
        {
            _isKeyboard = 0;
        }

        _horizontalAxis = context.ReadValue<float>() * (1 + keyboardMoveSpeed * _isKeyboard);
    }
}